#! /bin/bash
#
# written by SFr   www.frydaysoft.de (c) 2003

schematic() {
    echo "SCHEMATIC FOR THE PARALLEL PORT INTERFACE CIRCUIT"
    echo ""
    echo "NOTE: Power comes from the JTAG connector and feeds"
    echo "      just the 74HC240 and the pullups. The 74HC240 works OK"
    echo "      at 3.3 Volts. Instead of a 74HC240, a pair of 74HC14's"
    echo "      can also be used. Put also some decoupling on VDD"
    echo "      (10 uF will do)."
    echo ""
    echo "  JTAG VDD  ---------*------------------------*---  VDD"
    echo "  (PIN 1+2)          |                        |      (from JTAG)"
    echo "                74HC240 VDD                   |"
    echo "                                              |"
    echo "                                            [5K]"
    echo "  JTAG TMS  ________/|______/|_________[22K]__|___  D0 (LPT1)"
    echo "   (PIN 7)          \|      \|                       (PIN 2)"
    echo "                74HC240    74HC240"
    echo "                                             VDD"
    echo "                                              |"
    echo "                                             [5K]"
    echo "  JTAG TCK  ________/|______/|_________[22K]__|___  D1 (LPT1)"
    echo "   (PIN 9)          \|      \|                       (PIN 3)"
    echo "                 74HC240    74HC240"
    echo "                                             VDD"
    echo "                                              |"
    echo "                                             [5K]"
    echo "  JTAG TDI  ________/|______/|_________[22K]__|___  D2 (LPT1)"
    echo "   (PIN 5)          \|      \|                       (PIN 4)"
    echo "                 74HC240    74HC240"
    echo ""
    echo "  JTAG TDO  ________|\______|\____________________  SELECT (LPT1)"
    echo "   (PIN 13)         |/      |/                       (PIN 13)"
    echo "                 74HC240    74HC240"
    echo ""
    echo "  JTAG GND  ---------*-----*------*---------------  GND  (LPT1)"
    echo "   (PIN 20)          |     |      |                  (PINS 18..25)"
    echo "               74HC240 E1  |  74HC240 E2"
    echo "                           |"
    echo "                       74HC240 GND"
}

usage() {
    echo "Usage: $0 r{ead}   adr words {filename} ... read from target"
    echo "       $0 w{rite}  adr value            ... write a single word to target"
    echo "       $0 w{rite}  adr words filename   ... write file contents to target"
    echo "       $0 x{ecute} adr stackpointer     ... execute program at 'adr'"
    echo "       $0 s{chematic}                   ... display schematic"
    echo "       $0 v{ersion}                     ... print version of driver"
    echo ""
    echo "NOTE:  export ARMTOOL_SPEED=0	 if not set --> default max. speed (0)"
    echo ""
    echo "       This driver uses the JTAG i/o routines written by R.Longo / E.Authried."
    echo "                                          linux-driver   -   www.frydaysoft.de"
}


if [ -f /proc/armtool/version ]; then

    case "$1" in
    r)
	if [ "$#" = "3" ]; then
	    echo "$2">/proc/armtool/addr
	    cat </proc/armtool/value
        else
    	    if [ "$#" = "4" ]; then
		echo "$2">/proc/armtool/addr
	        echo "$3">/proc/armtool/size
	        cp /proc/armtool/data "$4"
	        cat </proc/armtool/info
	    else
    	        echo "wrong parameter"
	    fi
	fi
	exit
    ;;
    w)
	if [ "$#" = "3" ]; then
	    echo "$2">/proc/armtool/addr
	    echo "$3">/proc/armtool/value
	    cat </proc/armtool/info
        else
    	    if [ "$#" = "4" ]; then
		echo "$2">/proc/armtool/addr
	        echo "$3">/proc/armtool/size
	        cp "$4" /proc/armtool/data
	        cat </proc/armtool/info
	    else
    	        echo "wrong parameter"
	    fi
	fi
	exit
    ;;
    x)
	if [ "$#" = "3" ]; then
    	    echo "$2">/proc/armtool/addr
	    echo "$3">/proc/armtool/stack
    	    echo "1">/proc/armtool/exec
    	    cat </proc/armtool/info
	else
	    echo "wrong parameter"
	fi
	exit
    ;;
    s)
	schematic
	exit
    ;;
    v)
	cat </proc/armtool/version
	exit
    ;;
    *)
	usage
	exit
    ;;
    esac
else
    echo "driver not loaded"
    ioports=$(cat /proc/ioports | grep "^0378")
    if [ "$ioports" = "" ]; then
	echo "-> load driver with: insmod armdrv.o"
    else
	echo "check /proc/ioports: $ioports"
	echo "-> there is another driver at 0x378"
	echo "-> unload this driver at first"
	echo "e.g: rmmod lp"
	echo "     rmmod parport_pc"
    fi
    exit
fi
