#include <stdlib.h>
#include <stdio.h>
#include "jtag_io.h"
#include "proto.h"

//#define STOP_RUNING
#define RUN_PROGRAM

int verbose=0;	//set to non-zero for more info-prints
int DELAY=0;	//0=max.speed

void Sleep(int x)
{
    int i;
    for(i=0;i<x;i++) usleep(100);
}

void print_schematic()
{
	printf("\nSCHEMATIC FOR THE PARALLEL PORT INTERFACE CIRCUIT\n\n");

	printf("NOTE: Power comes from the JTAG connector and feeds\n");
	printf("      just the 74HC240 and the pullups. The 74HC240 works OK\n");
	printf("      at 3.3 Volts. Instead of a 74HC240, a pair of 74HC14's\n");
	printf("      can also be used. Put also some decoupling on VDD\n");
	printf("      (10 uF will do).\n\n");

	printf("\n  JTAG VDD  ---------*------------------------*---  VDD\n");
	printf("  (PIN 1+2)          |                        |      (from JTAG)\n");
	printf("                74HC240 VDD                   |\n");
	printf("                                              |\n");
	printf("                                            [5K]\n");
	printf("  JTAG TMS  ________/|______/|_________[22K]__|___  D0 (LPT1)\n");
	printf("   (PIN 7)          \\|      \\|                       (PIN 2)\n");
	printf("                74HC240    74HC240\n");
	printf("                                             VDD\n");
	printf("                                              |\n");
	printf("                                             [5K]\n");
	printf("  JTAG TCK  ________/|______/|_________[22K]__|___  D1 (LPT1)\n");
	printf("   (PIN 9)          \\|      \\|                       (PIN 3)\n");
	printf("                 74HC240    74HC240\n");
	printf("                                             VDD\n");
	printf("                                              |\n");
	printf("                                             [5K]\n");
	printf("  JTAG TDI  ________/|______/|_________[22K]__|___  D2 (LPT1)\n");
	printf("   (PIN 5)          \\|      \\|                       (PIN 4)\n");
	printf("                 74HC240    74HC240\n\n");
	printf("  JTAG TDO  ________|\\______|\\____________________  SELECT (LPT1)\n");
	printf("   (PIN 13)         |/      |/                       (PIN 13)\n");
	printf("                 74HC240    74HC240\n\n");
	printf("  JTAG GND  ---------*-----*------*---------------  GND  (LPT1)\n");
	printf("   (PIN 20)          |     |      |                  (PINS 18..25)\n");
	printf("               74HC240 E1  |  74HC240 E2\n");
	printf("                           |\n");
	printf("                       74HC240 GND\n");
}



int main(int argc,char **argv)
{
	u_int32_t adr,words;
	u_int32_t buf[16];
	unsigned char rolly[5]={'-','\\','|','/','\0'};
	long rollyCount=0;
	FILE *f;
	int n,i;
	
	char *p=0;
	int val=0;
	
	p=getenv("ARMTOOL_SPEED");
	if (p)
	{
	    if (sscanf(p,"%d",&val)==1)
	    DELAY=val;
	}
	
	if (argc<3) 
	{

		if (argc==2) 
		{
			if ( ( argv[1][0]=='s') || ( argv[1][0]=='S') )
			{
				print_schematic();
				//exit(1);
				return(1);
			}
		}

		printf("Usage: %s r{ead}  adr words {filename} ... read from target\n"
						"       %s w{rite} adr value            ... write a single word to target\n"
						"       %s w{rite} adr words filename   ... write file contents to target \n"
						"       %s x{ecute} adr stackpointer    ... execute program at 'adr'\n"
						"       %s s{chematic}                  ... display schematic\n\n"
						"NOTE:  export ARMTOOL_SPEED=%d\t if not set --> default max. speed (0)\n"
						"\n\n       This program uses the JTAG i/o routines written by R.Longo / E.Authried .\n"
						"\n\t\t\t\t\tlinux-binary   -   www.frydaysoft.de\n",
				argv[0],argv[0],argv[0],argv[0],argv[0], DELAY);
		//exit(1);
		return(1);
	}

	n=sscanf(argv[2],"%lu",&adr);
	if (n && adr==0 && argv[2][1]=='x') n=0;
	if (!n) n=sscanf(argv[2],"%lxu",&adr);
	if (n!=1) {fprintf(stderr,"argument error: adr\n"); return(0);}  //exit(1);}

	words=0;
	if (argc>3)
	{
		n=sscanf(argv[3],"%lu",&words);
		if (words==0)  n=0;
		if (!n) n=sscanf(argv[3],"%lxu",&words);
		if (n!=1) {fprintf(stderr,"argument error: words/stackpointer\n"); return(1); }  //exit(1);}
		printf("adr=0x%lx words=0x%x\n",adr,words);
	}

	initialize_jtag_hardware();
	ResetTapC();
	printf ("IDCODE=%08X\n", ReadIDCode());

	// ------------------------------------------------------------------------

#ifdef STOP_RUNING
	StopRunningProgram();
	Sleep(1000);
	if (verbose) printf("use StopRunningProgram()\n");
	// or next 6 lines
#endif
#ifndef STOP_RUNING
	PutBreakPoint(0x02000000);          // Put 'any address' brekpt.-> stop NOW!
	Sleep(100);
	if (verbose) printf("use PutBreakPoint()\n");
	ResetTapC();
	cpu_reset(0);				// Core is stopped now, release rst
	ResetTapC();
	ClearAnyBreakPoint();              // Avoid entering debug state after exiting 
#endif
	// ------------------------------------------------------------------------

	ResetTapC();
	total_ins_exec = 0;                // From now on, count instructions 

	rollyCount=0;
	switch(argv[1][0])
	{
		case 'w':
		case 'W':
			if (argc==5)
			{
				f=fopen(argv[4],"rb");
				if (f==NULL) {fprintf(stderr,"can't open file\n"); return(1);}  //exit(1);}
				printf("%c",rolly[rolly[5]%4]);
				while(words)
				{
					if (++rollyCount%0x20==0)
					{
						printf("%c",0x0D); //backspace
						printf("%c",rolly[++rolly[5]%4]);
					}
					n=words; if (n>14) n=14;
					n=fread(buf,4,n,f);
					if (n==0)  break;
					WriteMemoryBuf(adr,n,buf);
					adr+=4*n; words-=n;
				}
				printf("%c",0x0D); //backspace
				fclose(f); 
				fprintf(stderr, "end of file at 0x%lx\n",adr); 
				// exit(0);
				return(0);
			}
			else
			{
				WriteMemoryBuf(adr,1,&words);
			}
		break;

		case 'r':
		case 'R':
				f=NULL;
				if (argc==5)
				{
						f=fopen(argv[4],"wb");
						if (f==NULL) 
						{
							fprintf(stderr,"can't create file");
							//exit(1);
							return(1);
						}
				}
				ReadMemory(adr,words,f);
				if (f!=NULL) fclose(f);
				break;

		case 'x':
		case 'X':

				for (i=0; i<16; i++)
				{
					CPU_regs[i]=(i+1)*0x1000000;
					  switch(i)
					  {
						  case 13:
							  CPU_regs[i]=0x400;
							  break;
						  case 15:
							  CPU_regs[i]=0x2000000;
							  break;
					  }
				}

				if (words==0) words=0x400;

				WriteCpuRegs(words /*sp*/, adr /*pc*/, 1 /*overwrite CPU_regs[]*/);
				total_ins_exec = 0;                // From now on, count instructions 
				ResetTapC();					   // reset Tap-Controller
				ReadCpuRegs(0);                    // Read CPU registers & store away 
				ResetTapC();					   // reset Tap-Controller
				PrintRegs();					   // print CpuRegs & cpsr
				#ifdef RUN_PROGRAM
					RunProgram(adr);
					if (verbose) printf("use RunProgram()\n");
				#endif
				#ifndef RUN_PROGRAM
					if (verbose) printf("use ExitDebug()\n");
					// CPU_regs[15]=adr; total_ins_exec = 0; // use it if total_ins_exec wrong value
					ExitDebug();
				#endif
				
				break;
		} /* end switch */

		return 0;
}
