/*
read data from target
cp /proc/armtool/data file.bin
write data to target
cp file.bin /proc/armtool/data

todo:
write one instr
read
execute (print regs)
*/
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/ptrace.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/spinlock.h>
#include <linux/module.h>

#include <linux/proc_fs.h>
#include <linux/ip.h>
#include <linux/types.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#include "proto.c"
#include "jtag_io.c"

#define DATA 0
#define ADDR 1
#define SIZE 2
#define STACK 3
#define VERSION 4
#define EXEC 5
#define DELAY_VAL 6
#define VALUE 7
#define FULL_INFO 8

#define sPROC_ROOT "armtool"

#define sDATA "data"
#define sADDR "addr"
#define sSIZE "size"
#define sSTACK "stack"
#define sVERSION "version"
#define sEXEC "exec"
#define sDELAY_VAL "delay"
#define sVALUE "value"
#define sFULLINFO "info"

#define sVERSION_INFO "ARM7TDMI JTAG driver V1.0 www.frydaysoft.de (c)2003\n"

static int base=0x378;
MODULE_PARM(base, "i");
MODULE_PARM_DESC(base, "DATA_PORT [0x378 | 0x278 ...] (default is 0x378)");
#define HW_PARAM_INFO "load me: insmod drv DATA_PORT=LptPort\n"

static spinlock_t spinLock;

static int DELAY=0;
static int verbose=0;
#define MAXINFO 2048
static char infoBuffer[MAXINFO+1];
static int infoBufferPtr;

struct my_proc_struct{
	struct proc_dir_entry	*procRoot;
	struct proc_dir_entry	*procData;
	struct proc_dir_entry	*procAddr;
	struct proc_dir_entry	*procSize;
	struct proc_dir_entry	*procStack;
	struct proc_dir_entry	*procVersion;
	struct proc_dir_entry	*procExec;
	struct proc_dir_entry	*procDelay;
	struct proc_dir_entry	*procValue;
	struct proc_dir_entry	*procFullInfo;
	//here setting-datas
	long size;
	long addr;
	long stack;
}proc_data;


void clearInfoBuffer(void)
{
    int x;
    for(x=0;x<MAXINFO;x++)
	infoBuffer[x]=0;
    infoBufferPtr=0;
}

void printStr(char *str)
{
    if (str)
    {
	while(*str)
	{
	    if (infoBufferPtr>MAXINFO) return;
	    infoBuffer[infoBufferPtr++]=*str;
	    infoBuffer[infoBufferPtr]=0;   str++;
	}
    }
}

void printHex32(int val)
{
    int shift=7*4;
    char part='0';
    while(shift>=0)
    {
	if (infoBufferPtr>MAXINFO) return;
	part=(char)((val>>shift)&0x0000000F);
	if (part>=0xA) { part-=0xA; part+='A'; }
	else part+='0';
	shift-=4;
	infoBuffer[infoBufferPtr++]=part;
	infoBuffer[infoBufferPtr]=0;
    }
}

void printDez(int val)
{
    char reverse[11];
    int reversCnt=0;
    int res=val, part=0;
    while(res)
    {
	part=(res%10);
	reverse[reversCnt++]=(part+'0');
	res/=10;
    }
    if (!reversCnt) reverse[reversCnt++]='0';
    while (reversCnt)
    {
	if (infoBufferPtr>MAXINFO) return;
	infoBuffer[infoBufferPtr++]=reverse[--reversCnt];
	infoBuffer[infoBufferPtr]=0;
    }
}


void printSpace(int cnt)
{
    while(cnt)
    {
	if (infoBufferPtr>MAXINFO) return;
	infoBuffer[infoBufferPtr++]=' ';
	infoBuffer[infoBufferPtr]=0;
	cnt--;
    }
}

void Sleep(int x)
{
    int i;
    for(i=0;i<x;i++) usleep(100);
}

//#define STOP_RUNING
#define RUN_PROGRAM

void targetInit(int printID)
{
	int code=0;
	
	clearInfoBuffer();

	initialize_jtag_hardware();

	ResetTapC();
	code=ReadIDCode();
	if (printID)
	{
	    printStr("\nIDCODE=");  printHex32(code); printStr("\n");
	}
	// ------------------------------------------------------------------------

#ifdef STOP_RUNING
	StopRunningProgram();
	Sleep(1000);
	if (verbose) DPRINTK("use StopRunningProgram()\n");
	// or next 7 lines
#endif
#ifndef STOP_RUNING
	PutBreakPoint(0x02000000);          // Put 'any address' brekpt.-> stop NOW!
	Sleep(100);
	if (verbose) DPRINTK("use PutBreakPoint()\n");
	ResetTapC();
	cpu_reset(0);				// Core is stopped now, release rst
	ResetTapC();
	ClearAnyBreakPoint();              // Avoid entering debug state after exiting
#endif
	// ------------------------------------------------------------------------
	ResetTapC();
	total_ins_exec = 0;                // From now on, count instructions
}

void targetExec(int addr, int stack)
{
	int i;
	
	targetInit(1);
	
	if (!stack) stack=0x400;

	//init defaults for cpu registers
	for (i=0; i<16; i++)
	{
		CPU_regs[i]=(i+1)*0x1000000;
		switch(i)
		{
			case 13:
				CPU_regs[i]=stack;
				break;
			case 15:
				CPU_regs[i]=addr;
				break;
		}
	}
	WriteCpuRegs(stack, addr, 1);	//overwrite these regs
	total_ins_exec = 0;     // from now on, count instructions
	ResetTapC();		// reset Tap-Controller
	ReadCpuRegs(0);		// read CPU registers & store away
	ResetTapC();		// reset Tap-Controller
	PrintRegs();		// print CpuRegs & cpsr
#ifdef RUN_PROGRAM
		RunProgram(addr);
		if (verbose) DPRINTK("use RunProgram()\n");
#endif
#ifndef RUN_PROGRAM
		if (verbose) DPRINTK("use ExitDebug()\n");
		// CPU_regs[15]=adr; total_ins_exec = 0; // use it if total_ins_exec has a wrong value
		ExitDebug();
#endif
}


//****************************************************************************************
// convert string to (hex)number
long my_strtol(char* buf, int len)
{
    int pos=len;
    long ret=0;
    long mul=1;
    long multype=10;

    if (pos) pos--;	//we need not '/0'

    if (pos)
    {
	if (pos>2)  {
	    if (buf[1]=='x') {
		multype=16;
	    }
	}
	for(pos--;pos!=-1;pos--) //we need not '/n'
	{
	    if (multype==16) if (pos==1) break;
	    if ( (buf[pos]<='9') && ( buf[pos]>='0') ) 	    {
		ret+=(buf[pos]-'0')*mul;	//'0'..'9'
	    }
	    else    {
		ret+=((buf[pos]&0x7)+9)*mul; 	//'A'/'a'..'F'/'f'
	    }
	    mul*=multype;
	}
    }
    return ret;
}

//****************************************************************************************
// store settings into driver
int readProc(struct file *file, const char *buf, unsigned long count, void *data, int callType)
{
    int addr=0;
    long val=0;
    int len=count, part=0, offs=0;
#ifdef USE_COPY
    unsigned int _buf[16];
#endif
    
#ifdef DBG_FUNC_NAME
    DPRINTK(" <<drv.c: " __FUNCTION__ " <<\n");
#endif
    spin_lock(&spinLock);
    switch(callType)
    {
	case DATA:
		//default: in 4096 byte pages
		if (file->f_pos==0)
		{
		  targetInit(0);
		  printStr("write to: 0x"); printHex32(proc_data.addr); printStr("\n");
		}
		addr=(proc_data.addr+file->f_pos);
		len=(int)((count+3)/4);	offs=0;
		//DPRINTK("%ld WRITE:  pos:%d addr:%X len:%d intLen:%X \n",jiffies, (int)file->f_pos, (int)addr, (int)count, len);
		while(len>0)
		{
		    part=len;    if (part>14) part=14;
#		    ifdef USE_COPY		    
			copy_from_user(_buf,buf+offs,part*4);
			WriteMemoryBuf(addr, part, _buf);
#		    else			
			WriteMemoryBuf(addr, part, (int*)(buf+offs));
#		    endif
		    offs+=(part*4);    addr+=(part*4);
		    len-=part;
		}
		file->f_pos+=count;
		break;
	case ADDR:
		len&=0xF;
		val=my_strtol((char*)buf,len);
		proc_data.addr=val;
		break;
	case SIZE:
		len&=0xF;
		val=my_strtol((char*)buf,len);
		proc_data.size=val;
		break;
	case STACK:
		len&=0xF;
		val=my_strtol((char*)buf,len);
		proc_data.stack=val;
		break;
	case VERSION:
		DPRINTK(" funcCall proc version \n");
		break;
	case EXEC:
		targetExec(proc_data.addr, proc_data.stack);
		break;
	case DELAY_VAL:
		len&=0xF;
		val=my_strtol((char*)buf,len);
		DELAY=val;
		break;
	case VALUE:
		targetInit(0);
		len&=0xF;
		val=my_strtol((char*)buf,len);
		WriteMemoryBuf(proc_data.addr, 1, (int*)&val);
		printStr("write 0x"); printHex32(val); printStr(" to 0x");  printHex32(proc_data.addr); printStr("\n");		
		break;
    }
    spin_unlock(&spinLock);
    return count;
}


//****************************************************************************************
//write settings back to proc-system
int writeProc(char *page, char **start, off_t off, int count, int *eof, void *data, int callType)
{
    int ret=0;
    int len=0;
    int pos=0;
    int offs=(off+3)/4;
    int pageCount=(count+3)/4;
    long val=0;
#ifdef DBG_FUNC_NAME    
    DPRINTK(" <<drv.c: " __FUNCTION__ " <<\n");
#endif
    spin_lock(&spinLock);
    switch(callType)
    {
	case DATA:
	//xxx
		// default: 0xc00 bytes in the page
		if (!off)
		{
		    targetInit(0);
		}
		len=proc_data.size-offs;
		if (len>pageCount) len=pageCount;
		*eof=(len+offs)<proc_data.size?0:1;
		if (len)
		{
		    printStr("\nread from: 0x"); printHex32(proc_data.addr); printSpace(1);
		    printDez(len); printStr(" longs\n");
		    ReadMemory(proc_data.addr+off, len, (int*)page);
		}
		*start=page;
		
		ret=len*4;
		printStr("return:"); printDez(ret);
		break;
	case ADDR:
		ret=sprintf(page,"Addr=0x%lX\n", proc_data.addr);
	        *eof=1;
		break;
	case SIZE:
		ret=sprintf(page,"Size=0x%lX\n", proc_data.size);
	        *eof=1;
		break;
	case STACK:
		ret=sprintf(page,"Stack=0x%lX\n", proc_data.stack);
	        *eof=1;
		break;
	case VERSION:
		ret=sprintf(page,sVERSION_INFO);
	        *eof=1;
		break;
	case EXEC:
		ret=sprintf(page,"use cat < exec !!!\n");
	        *eof=1;
		break;
	case DELAY_VAL:
		ret=sprintf(page,"DELAY=%d\n", DELAY);
		break;
	case VALUE:
		targetInit(0);
		ReadMemory(proc_data.addr, 1, (int*)&val);
		ret=sprintf(page,"%lX\n", val);
		break;
	case FULL_INFO:
		if (infoBufferPtr)
		{
		    len=infoBufferPtr-off;
		    *eof=len>count?0:1;
		    if (len>count) len=count;
		    for (pos=0; pos<len; pos++)
		      page[pos]=infoBuffer[off+pos];
		    *start=page;
		    ret=len;		
		}
		else
		{
		    ret=sprintf(page,"Base: 0x%X\nAddr=0x%lX\nSize=0x%lX\nStack=0x%lX\n\n", DATA_PORT, proc_data.addr, proc_data.size, proc_data.stack);
	    	    *eof=1;
		}
		break;
    }
    spin_unlock(&spinLock);
    return ret;
}

//****************************************************************************************
// functions for procsystem
int getData(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, DATA);}
int getAddr(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, ADDR);}
int getSize(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, SIZE);}
int getStack(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, STACK);}
int getVersion(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, VERSION);}
int getExec(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, EXEC);}
int getDelay(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, DELAY_VAL);}
int getValue(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, VALUE);}
int getFullInfo(char *page, char **start, off_t off, int count, int *eof, void *data)
    {    return writeProc(page, start, off, count, eof, data, FULL_INFO);}

int setData(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, DATA);}
int setAddr(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, ADDR);}
int setSize(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, SIZE);}
int setStack(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, STACK);}
int setVersion(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, VERSION);}
int setExec(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, EXEC);}
int setDelay(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, DELAY_VAL);}
int setValue(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, VALUE);}
int setFullInfo(struct file *file, const char *buf, unsigned long count, void *data)
    {    return readProc(file, buf, count, data, FULL_INFO);}

/* ------------------------------------------------------------------------ */

static void __exit drv_exit(void)
{
//#ifdef DBG_FUNC_NAME    
    DPRINTK(" <<drv.c: " __FUNCTION__ " <<\n");
//#endif
    
    remove_proc_entry(sDATA, proc_data.procRoot);
    remove_proc_entry(sADDR, proc_data.procRoot);
    remove_proc_entry(sSIZE, proc_data.procRoot);
    remove_proc_entry(sSTACK, proc_data.procRoot);
    remove_proc_entry(sVERSION, proc_data.procRoot);
    remove_proc_entry(sEXEC, proc_data.procRoot);
    remove_proc_entry(sDELAY_VAL, proc_data.procRoot);
    remove_proc_entry(sVALUE, proc_data.procRoot);
    remove_proc_entry(sFULLINFO, proc_data.procRoot);
    remove_proc_entry(sPROC_ROOT, NULL);
    
    release_region(DATA_PORT, 3);
}

/* ------------------------------------------------------------------------ */

static int __init drv_init(void)
{
#ifdef DBG_FUNC_NAME    
    DPRINTK(" <<drv.c: " __FUNCTION__ " <<\n");
#endif
    DATA_PORT=base;
    STATUS_PORT=base+1;

    if (check_region(DATA_PORT, 3))
    {
	//DPRINTK("i/o busy\n");
	return -EBUSY;
    }
    //DPRINTK("i/o free\n");
    
    spin_lock_init(&spinLock);

    // register into /proc/ioports    
    request_region(DATA_PORT, 3, "armdrv");

//    DPRINTK("Base=%x %s\n",DATA_PORT, sVERSION_INFO);

//init proc-entries:
    proc_data.procRoot=create_proc_entry(sPROC_ROOT, 0777|S_IFDIR, NULL);
    if (!proc_data.procRoot) DPRINTK("proc-root bad\n");

    proc_data.procData=create_proc_entry(sDATA, 0777|S_IFREG, proc_data.procRoot);
    if(!proc_data.procData) DPRINTK("data entry bad\n");
    proc_data.procData->read_proc=getData;
    proc_data.procData->write_proc=setData;

    proc_data.procAddr=create_proc_entry(sADDR, 0777|S_IFREG, proc_data.procRoot);
    if(!proc_data.procAddr) DPRINTK("addr entry bad\n");
    proc_data.procAddr->read_proc=getAddr;
    proc_data.procAddr->write_proc=setAddr;
    proc_data.addr=0x2000000;

    proc_data.procSize=create_proc_entry(sSIZE, 0777|S_IFREG, proc_data.procRoot);
    if(!proc_data.procSize) DPRINTK("size entry bad\n");
    proc_data.procSize->read_proc=getSize;
    proc_data.procSize->write_proc=setSize;
    proc_data.size=0;

    proc_data.procStack=create_proc_entry(sSTACK, 0777|S_IFREG, proc_data.procRoot);
    if(!proc_data.procStack) DPRINTK("stack entry bad\n");
    proc_data.procStack->read_proc=getStack;
    proc_data.procStack->write_proc=setStack;
    proc_data.stack=0x400;

    proc_data.procVersion=create_proc_entry(sVERSION, 0655|S_IFREG, proc_data.procRoot);
    if(!proc_data.procVersion) DPRINTK("version entry bad\n");
    proc_data.procVersion->read_proc=getVersion;
    proc_data.procVersion->write_proc=setVersion;

    proc_data.procExec=create_proc_entry(sEXEC, 0777|S_IFREG, proc_data.procRoot);
    if(!proc_data.procExec) DPRINTK("exec entry bad\n");
    proc_data.procExec->read_proc=getExec;
    proc_data.procExec->write_proc=setExec;

    proc_data.procDelay=create_proc_entry(sDELAY_VAL, 0777|S_IFREG, proc_data.procRoot);
    if(!proc_data.procDelay) DPRINTK("exec entry bad\n");
    proc_data.procDelay->read_proc=getDelay;
    proc_data.procDelay->write_proc=setDelay;

    proc_data.procValue=create_proc_entry(sVALUE, 0777|S_IFREG, proc_data.procRoot);
    if(!proc_data.procValue) DPRINTK("exec entry bad\n");
    proc_data.procValue->read_proc=getValue;
    proc_data.procValue->write_proc=setValue;

    proc_data.procFullInfo=create_proc_entry(sFULLINFO, 0655|S_IFREG, proc_data.procRoot);
    if(!proc_data.procFullInfo) DPRINTK("info entry bad\n");
    proc_data.procFullInfo->read_proc=getFullInfo;
    proc_data.procFullInfo->write_proc=setFullInfo;
   return 0;
}

//**************************************************************************

module_init(drv_init);
module_exit(drv_exit);

/* ------------------------------------------------------------------------ */


MODULE_AUTHOR("www.frydaysoft.de (c)2003");
MODULE_DESCRIPTION("parallel JTAG driver for ARM7TDMI");
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif
EXPORT_NO_SYMBOLS;

