#include "proto.h"
#include "jtag_io.h"

int DATA_PORT=0x378;
int STATUS_PORT=0x379;

unsigned char dataPortImg = 0;   /* Local copy of data port */
int f_reset;

int n_c0=0;	// chain before AT91
int l_c0=0;

int n_c1=0;	// nr. of devs after AT91
int l_c1=0;	// sum of IRs after AT91

void speedwait(int count)
{
    int x;
    for (x=0; x<count; x++) { x++;x--;}
}

/*
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

static int start_uid = 0;
static int start_euid = 0;
#define priv_init() { start_uid = (int)getuid(); start_euid = (int)geteuid(); }
#define priv_root() seteuid(0)
#define priv_revoke() seteuid(start_uid)

     priv_init()
     printf("UID=%d, EUID=%d\n", start_uid, start_euid);
     priv_root();
     ioperm(DATA_PORT,3,1);
     ....
     priv_revoke();
*/

/* Parallel port initialization */
void initialize_jtag_hardware(void)
{
#ifdef USE_MAIN
	char *p;
	int e;
     fprintf(stderr,"initialize_jtag_hardware()\n");

     //e=ioperm(DATA_PORT,8,1);
     e=ioperm(DATA_PORT,3,1);
     if (e==-1)
     {
        printf("ioperm(%d)  -->  you need root-access\n",e);
	INB(STATUS_PORT); //-->generates a memory-exception
	printf("exception inb\n");
	//exit(0);
     }
#endif     
     dataPortImg=0xff;
     cpu_reset(0);		// release reset

     jam_jtag_io(0,0,0);

     n_c0=0;
     l_c0=0;
     n_c1=0;
     l_c1=0;
     return;

     // maybe used in future
     /*
     p = getenv("JTAG_CHAIN");
     if (!p) return;

     e=sscanf(p,"%d %d %d %d",&n_c0,&l_c0,&n_c1,&l_c1);
     if (e!=4) {
		fprintf(stderr, "Data error in JTAG_CHAIN: %s\n", p);
     }
     fprintf(stderr,"JTAG_CHAIN=%d %d %d %d\n",n_c0,l_c0,n_c1,l_c1);
     */
}

void cpu_reset(int rst)
{
  f_reset=(rst)?MASK_RST_BIT:0;	
  jam_jtag_io(0,0,0);
}

/*! \brief JTAG i/o
    \param tms output value for TMS
    \param tdi output value for TDI
    \param read_tdo 1:read  and return TDO, 0:return 0
 */
int jam_jtag_io(int tms, int tdi, int read_tdo)
{
	int data=0;
	int tdo=0;

	/*
	static int is_initialized=0;
	if (!is_initialized) 
	{
		is_initialized=1;
		initialize_jtag_hardware();
	}
	*/
	//DPRINTK("io-access\n");

	
	data = (tdi ? (INVERT_TDI?0:MASK_TDI_BIT) : (INVERT_TDI?MASK_TDI_BIT:0))
			| (tms ? (INVERT_TMS?0:MASK_TMS_BIT) : (INVERT_TMS?MASK_TMS_BIT:0))
			| f_reset;
    if (data!=dataPortImg)
	{
		OUTB(data | ((INVERT_TCK)?MASK_TCK_BIT:0),DATA_PORT);	// TMS,TDI
		dataPortImg=data;
		if (DELAY) speedwait(DELAY);
	}

    if (read_tdo) 
	{
		tdo = (INB(STATUS_PORT)&MASK_TDO_BIT) ? (INVERT_TDO?0:1):(INVERT_TDO?1:0);                // Input bit
		if (DELAY) speedwait(DELAY);
	}

		// TCK pulse
    OUTB(data|((INVERT_TCK)?0:MASK_TCK_BIT) ,DATA_PORT);   	// tck
	if (DELAY) speedwait(DELAY);
    OUTB(data|((INVERT_TCK)?MASK_TCK_BIT:0),DATA_PORT);     // set tck to 0
	if (DELAY) speedwait(DELAY);

#ifdef USE_MAIN
    if (verbose)
    {
        fprintf(stderr,"tms/tdi %d/%d ",tms!=0,tdi!=0);
        if (read_tdo) 
	    fprintf(stderr,"tdo=%d",tdo);
        fprintf(stderr,"\n");
    }
#endif

    return (tdo);
}

void close_jtag_hardware(void)
{
    return ;
}

#ifdef USE_MAIN
void usleep(int microseconds)
{
	//struct timeval s;
	//s.tv_sec = 0; 
    	//s.tv_usec = microseconds;
    	//select( 0, NULL, NULL, NULL, &s );

	//runs also in win32
	int x;
	for (x=0; x<10*microseconds; x++) INB(STATUS_PORT);
}	
#endif

