#ifndef PROTO_H
#define PROTO_H

#include "machine.h"

	extern u_int32_t CPU_regs[16];            /* 16 CPU regs image (R0..R15) */
	extern u_int32_t CPU_CPSR;                /* CPSR image */

	extern int n_c0;	// chain before AT91
	extern int l_c0;
	extern int n_c1;	// nr. of devs after AT91
	extern int l_c1;	// sum of IRs after AT91

	extern int was_in_thumb;                     // THUMB mode flag             
	extern int total_ins_exec;                   // total # of instr. executed  
	extern int is_watchpoint;                    // Breakpoint=0, watchpoint=1  
	extern int is_debugrequest;                  // User requested debug (not brkp.)

	extern int DELAY;

	/***  Reset the JTAG TAP controller      ***/
	void ResetTapC(void);

	/***  Step to next state in TAP machine  ***/
	void StepNextTAPState(int newtms);

	/***  Enter a new instr. in TAP machine  ***/
	int EnterIRInst(int nbits, unsigned int newinst);

	void SC1(int b, u_int32_t instruction, u_int32_t *outdata);

	void RunTest_Idle(void);

	void EnterRegister(u_int32_t *pIn, u_int32_t *pOut, int c0_bits, int n_bits, int c1_bits, int tms_exit);


	/***  Enter a bit stream in the selected DR  ***/
	int EnterDRBits(int nbits, unsigned int *newdata, unsigned int *olddata, int enter_DR, int exit_DR);

	/***  Enter an instruction+BREAKPT bit in Scan chain 1  ***/
	void EnterSC1Bits(unsigned int *newdata, unsigned int *olddata);

	/***  Read the Device Identification code    ***/
	extern unsigned int ReadIDCode(void);

	int SelectScanChain(unsigned int sc);

	/***  Read out one IceBreaker register contents ***/
	unsigned int IceBreakerRegRead(int nreg);

	/***  Overwrite IceBreaker register contents    ***/
	unsigned int IceBreakerRegWrite(int nreg, unsigned int regdata);

	/***  Read & print ALL of IceBreaker registers  ***/
	int ReadAllIceBreakerRegs(void);

	/***  Stop a running program and enter DEBUG state  ***/
	int StopRunningProgram(void);


	/* Put a breakpoint at some address using ICEBreaker */
	int PutBreakPoint(unsigned int addr);

	/* Disable any enabled breakpoint */
	int ClearAnyBreakPoint(void);

	/* Clear Debug Request */
	int ClearUserDebugRequest(void);

	/***  Read out internal CPU registers (ARM state)   ***/
	int ReadCpuRegs(int reset_inst_counter);

	/***  write (restore) all CPU- registers ***/
	int WriteCpuRegs(u_int32_t sp, u_int32_t pc, bool copyIntoRegs);

	/***  Restore registers and RUN (exit DEBUG state)  ***/
	int ExitDebug(void);

	/***         Read memory while in DEBUG state       ***/
#ifdef USE_MAIN
	int ReadMemory(u_int32_t address, int howmanywords, FILE *f);
#else
	int ReadMemory(u_int32_t address, int howmanywords, int *page);
#endif


	u_int32_t ReadWord(u_int32_t address);

	void WriteMemoryLocation(u_int32_t address, u_int32_t value);

	void WriteMemoryBuf(u_int32_t address, int count, u_int32_t *pMemory);

	void RunProgram(u_int32_t address);

	void PrintRegs(void);
#endif
